/**********************************************************************************************/
/*                                                                                            */
/*                                     .                                        */
/*                                                                                            */
/**********************************************************************************************/
#include  "display.h"


/*--------------------------------------------------------------------------------------------*/
/*   .                                                           */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Display_Init(void)
{
  //       :
  //     ,  CS1 = "1".
  DDRC |= ((1 << 0) | (1 << 1) | (1 << 2) | (1 << 3) | (1 << 5));
  PORTC &= ~((1 << 0) | (1 << 1) | (1 << 2) | (1 << 5));
  PORTC |= (1 << 3);
  //  .   ,  
  //      .
  for(WORD i = 0; i < 1000; i++) Clr_Line_in_Port(LCD_RES_PIN);
  for(WORD i = 0; i < 1000; i++) Set_Line_in_Port(LCD_RES_PIN);
  for(WORD i = 0; i < 1000; i++) Set_Line_in_Port(LCD_RES_PIN);
  //  .
  Write_Cmd_LCD_Bias_Set(0);              //   - 1/9.
  Write_Cmd_ADC_Normal();                 //  -.
  Write_Cmd_Scan_Reverse();               //  - -.
  Write_Cmd_Display_Normal();             //   =  .
  Write_Cmd_Booster_Ratio(0);             //   booster =  .
  Write_Cmd_V5_Res_Ratio(4);              //  -   .
  Write_Cmd_Volume_Set(LCD_Bright);
  Write_Cmd_Power_Control_Set(1, 1, 1);   //    .
  Write_Cmd_Set_Start_Line(0);            //  -   .
  LCD_Clear();                            //  .
  Write_Cmd_Display_ON();                 //  .
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*      .                                                 */
/* :     .                                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Write_Cmd_to_LCD(IN BYTE cmd)
{
  //          .
  LCD_Data_Bus_to_Output();
  Clr_Line_in_Port(LCD_A0_PIN);
  Clr_Line_in_Port(LCD_RW_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_CS1_PIN);
  //          .
  Put_Data_to_LCD_Data_Bus(cmd);
  Set_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  //    .
  Set_Line_in_Port(LCD_CS1_PIN);
  LCD_Data_Bus_Off();
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   1     .                                          */
/* :     .                                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Write_Data_to_LCD(IN BYTE data)
{
  //          .
  LCD_Data_Bus_to_Output();
  Set_Line_in_Port(LCD_A0_PIN);
  Clr_Line_in_Port(LCD_RW_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_CS1_PIN);
  //          .
  Put_Data_to_LCD_Data_Bus(data);
  Set_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  //    .
  Set_Line_in_Port(LCD_CS1_PIN);
  LCD_Data_Bus_Off();
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                         */
/* :      ,   .                       */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Write_Data_to_LCD(IN BYTE __flash *pData, IN BYTE n)
{
  //          .
  LCD_Data_Bus_to_Output();
  Set_Line_in_Port(LCD_A0_PIN);
  Clr_Line_in_Port(LCD_RW_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_CS1_PIN);
  //   c        .
  for(BYTE i = 0; i < n; i++)
     {
       Put_Data_to_LCD_Data_Bus(*(pData + i));
       Set_Line_in_Port(LCD_E_PIN);
       Clr_Line_in_Port(LCD_E_PIN);
     }
  //    .
  Set_Line_in_Port(LCD_CS1_PIN);
  LCD_Data_Bus_Off();
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                         */
/* :      ,   .                       */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Write_Data_to_LCD(IN BYTE *pData, IN BYTE n)
{
  //          .
  LCD_Data_Bus_to_Output();
  Set_Line_in_Port(LCD_A0_PIN);
  Clr_Line_in_Port(LCD_RW_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_CS1_PIN);
  //   c        .
  for(BYTE i = 0; i < n; i++)
     {
       Put_Data_to_LCD_Data_Bus(*(pData + i));
       Set_Line_in_Port(LCD_E_PIN);
       Clr_Line_in_Port(LCD_E_PIN);
     }
  //    .
  Set_Line_in_Port(LCD_CS1_PIN);
  LCD_Data_Bus_Off();
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   n      .                                    */
/* :    ,   .                                         */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Write_Data_to_LCD(IN BYTE data, IN BYTE n)
{
  //          .
  LCD_Data_Bus_to_Output();
  Set_Line_in_Port(LCD_A0_PIN);
  Clr_Line_in_Port(LCD_RW_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_CS1_PIN);
  //   c         .
  for(BYTE i = 0; i < n; i++)
     {
       Put_Data_to_LCD_Data_Bus(data);
       Set_Line_in_Port(LCD_E_PIN);
       Clr_Line_in_Port(LCD_E_PIN);
     }
  //    .
  Set_Line_in_Port(LCD_CS1_PIN);
  LCD_Data_Bus_Off();
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   .                                                                     */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void LCD_Clear(void)
{
  for(register IDX page = 0; page < 8; page++)
     {
       //  .
       Write_Cmd_Page_Addr_Set(page);
       //    .
       Write_Cmd_Column_Addr_Set(0);
       Write_Data_to_LCD((BYTE)0, 128);
     }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   1     .                                        */
/* :     .                                                          */
/* :   .                                                               */
/*--------------------------------------------------------------------------------------------*/
__monitor OUT BYTE Read_Data_from_LCD(void)
{
  //          .
  LCD_Data_Bus_To_Input();
  Set_Line_in_Port(LCD_A0_PIN);
  Set_Line_in_Port(LCD_RW_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_CS1_PIN);
  //   - dummy.
  Set_Line_in_Port(LCD_E_PIN);
  __no_operation();
  Clr_Line_in_Port(LCD_E_PIN);
  //        .
  Set_Line_in_Port(LCD_E_PIN);
  __no_operation();         //     
  __no_operation();         //  NOP'  .
  __no_operation();
  BYTE data = Get_Data_from_LCD_Data_Bus();
  Clr_Line_in_Port(LCD_E_PIN);
  //    .
  Set_Line_in_Port(LCD_CS1_PIN);
  LCD_Data_Bus_Off();
  return data;
}

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                       */
/* :       ,   .                 */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Read_Data_from_LCD(OUT BYTE *pData, IN BYTE n)
{
  //          .
  LCD_Data_Bus_To_Input();
  Set_Line_in_Port(LCD_A0_PIN);
  Set_Line_in_Port(LCD_RW_PIN);
  Clr_Line_in_Port(LCD_E_PIN);
  Clr_Line_in_Port(LCD_CS1_PIN);
  //   - dummy.
  Set_Line_in_Port(LCD_E_PIN);
  __no_operation();
  Clr_Line_in_Port(LCD_E_PIN);
  //          .
  for(BYTE i = 0; i < n; i++)
     {
       Set_Line_in_Port(LCD_E_PIN);
       __no_operation();      //     
       __no_operation();      //  NOP'  .
       __no_operation();
       *(pData + i) = Get_Data_from_LCD_Data_Bus();
       Clr_Line_in_Port(LCD_E_PIN);
     }
  //    .
  Set_Line_in_Port(LCD_CS1_PIN);
  LCD_Data_Bus_Off();
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*       .                                  */
/* :      (),                                  */
/*                 ,                                                  */
/*                    flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Bitmap(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H, IN BYTE __flash *pData, IN BYTE flags)
{
  //     .
  BYTE buf[MAX_WIDTH];
  //   ,     .
  if((X >= MAX_WIDTH) || (Y >= MAX_HEIGHT)) return;
  //      .
  BYTE w = ((BYTE)(X + W) < MAX_WIDTH) ? (W) : (MAX_WIDTH - X);
  //       .
  BYTE h = H;   //       .
  while(h > 0)
    {
      //    ,    .
      //      .
      BYTE page = Y / 8;
      if(page >= 8) break;
      //     .
      Write_Cmd_Page_Addr_Set(page);
      Write_Cmd_Column_Addr_Set(X);
      Read_Data_from_LCD(&buf[0], w);
      //  ,      
      //        .
      BYTE mask = 0x00, n = 0;
      for(BYTE i = 0; i < 8; i++)
         {
           mask >>= 1;
           if(((BYTE)(((page * 8) + i)) >= Y) && ((BYTE)(((page * 8) + i)) < (BYTE)(Y + h)))
             {
               mask |= 0x80; n++;
             }
         }
      mask = ~mask;
      //         
      //       .
      if(((H - h) % 8) != 0) n = min(8 - ((H - h) % 8), n);
      //      .
      BYTE __flash *p = pData + W * ((H - h) / 8);
      for(BYTE i = 0; i < w; i++)
         {
           BYTE data = *(p + i);
           //    - .
           if(((flags & DRAW_UNDERLINED) != 0) && ((h - n) <= 0)) data |= (1 << (n - 1));
           //    - .
           if((flags & DRAW_INVERTED) != 0) data = ~data;
           //      .
           if((Y % 8) == 0) data >>= ((H - h) % 8);
           else             data <<= (Y % 8);
           //      .
           data &= ~mask;
           if((flags & DRAW_XOR) != 0) buf[i] ^= data;
           else                        buf[i] = (buf[i] & mask) | data;
         }
      //       .
      Write_Cmd_Page_Addr_Set(page);
      Write_Cmd_Column_Addr_Set(X);
      Write_Data_to_LCD(&buf[0], w);
      //     .
      h -= n; Y += n;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*     .                                       */
/* :         ,                 */
/*                   ,                                 */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Fill_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H, IN BYTE flags)
{
  //     .
  BYTE buf[MAX_WIDTH];
  //   ,     .
  if((X >= MAX_WIDTH) || (Y >= MAX_HEIGHT)) return;
  //       .
  if((BYTE)(X + W) > MAX_WIDTH) W = MAX_WIDTH - X;
  //        .
  BYTE h = H;   //       .
  while(h > 0)
    {
      //    ,    .
      //      .
      BYTE page = Y / 8;
      if(page >= 8) break;
      //     .
      Write_Cmd_Page_Addr_Set(page);
      Write_Cmd_Column_Addr_Set(X);
      Read_Data_from_LCD(&buf[0], W);
      //  ,      
      //        .
      BYTE mask = 0x00, n = 0;
      for(BYTE i = 0; i < 8; i++)
         {
           mask >>= 1;
           if(((BYTE)(((page * 8) + i)) >= Y) && ((BYTE)(((page * 8) + i)) < (BYTE)(Y + h)))
             {
               mask |= 0x80; n++;
             }
         }
      //      .
      for(BYTE i = 0; i < W; i++) buf[i] = ((flags & DRAW_INVERTED) != 0) ? (buf[i] & ~mask) : (buf[i] | mask);
      //       .
      Write_Cmd_Page_Addr_Set(page);
      Write_Cmd_Column_Addr_Set(X);
      Write_Data_to_LCD(&buf[0], W);
      //     .
      h -= n; Y += n;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*     .                                     */
/* :         ,                 */
/*                   .                                 */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Invert_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H)
{
  //     .
  BYTE buf[MAX_WIDTH];
  //   ,     .
  if((X >= MAX_WIDTH) || (Y >= MAX_HEIGHT)) return;
  //       .
  if((BYTE)(X + W) > MAX_WIDTH) W = MAX_WIDTH - X;
  //        .
  BYTE h = H;   //       .
  while(h > 0)
    {
      //    ,    .
      //      .
      BYTE page = Y / 8;
      if(page >= 8) break;
      //     .
      Write_Cmd_Page_Addr_Set(page);
      Write_Cmd_Column_Addr_Set(X);
      Read_Data_from_LCD(&buf[0], W);
      //  ,      
      //        .
      BYTE mask = 0x00, n = 0;
      for(BYTE i = 0; i < 8; i++)
         {
           mask >>= 1;
           if(((BYTE)(((page * 8) + i)) >= Y) && ((BYTE)(((page * 8) + i)) < (BYTE)(Y + h)))
             {
               mask |= 0x80; n++;
             }
         }
      //   .
      for(BYTE i = 0; i < W; i++) buf[i] = (buf[i] & ~mask) | (~buf[i] & mask);
      //       .
      Write_Cmd_Page_Addr_Set(page);
      Write_Cmd_Column_Addr_Set(X);
      Write_Data_to_LCD(&buf[0], W);
      //     .
      h -= n; Y += n;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*      .                                            */
/* :      (),                                        */
/*                 .                                                  */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Put_Pixel(IN BYTE X, IN BYTE Y, IN BYTE flags)
{
  //   ,     .
  if((X >= MAX_WIDTH) || (Y >= MAX_HEIGHT)) return;
  //   ,     .
  Write_Cmd_Page_Addr_Set(Y / 8);
  Write_Cmd_Column_Addr_Set(X);
  BYTE byte = Read_Data_from_LCD();
  //     .
  if((flags & DRAW_INVERTED) != 0) byte &= ~(1 << (Y % 8));
  else                             byte |=  (1 << (Y % 8));
  //     .
  Write_Cmd_Column_Addr_Set(X);
  Write_Data_to_LCD(byte);
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*       ( ).                */
/* :        (),                               */
/*                 .                                                */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Line(IN BYTE X1, IN BYTE Y1, IN BYTE X2, IN BYTE Y2, IN BYTE flags)
{
  //         .
  signed char dX = X2 - X1, dY = Y2 - Y1;
  //    (   X,    Y).
  signed char inc_x, inc_y;
  if(dX > 0) inc_x = 1; else if (dX == 0) inc_x = 0; else inc_x = -1;
  if(dY > 0) inc_y = 1; else if (dY == 0) inc_y = 0; else inc_y = -1;
  //    .       .
  dX = abs(dX);
  dY = abs(dY);
  BYTE distance = (dX > dY) ? (dX) : (dY);
  //  .
  for(BYTE i = 0, x = 0, y = 0; i <= distance + 1; i++)
     {
       if((X1 < MAX_WIDTH) && (Y1 < MAX_HEIGHT)) Put_Pixel(X1, Y1, flags);
       x += dX; y += dY;
       if(x > distance)
         {
           x -= distance;
           X1 += inc_x;
         }
       if(y > distance)
         {
           y -= distance;
           Y1 += inc_y;
         }
     }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*        .                          */
/* :      ,                                                           */
/*                 (),                                      */
/*                   flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Text(IN char *str, IN BYTE X, IN BYTE Y, IN struct FONT __flash *font, IN BYTE flags)
{
  //         (0x00).
  while(*str != 0)
    {
      //   .
      Draw_Char(*str, X, Y, font, flags);
      str++;
      X += font->Width;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*        .                          */
/* :      flash,                                                        */
/*                 (),                                      */
/*                   flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Text(IN char __flash *str, IN BYTE X, IN BYTE Y, IN struct FONT __flash *font, IN BYTE flags)
{
  //         (0x00).
  while(*str != 0)
    {
      //   .
      Draw_Char(*str, X, Y, font, flags);
      str++;
      X += font->Width;
    }
  return;
}

/**********************************************************************************************/
