/**********************************************************************************************/
/*                                                                                            */
/*                              .                                 */
/*                                                                                            */
/**********************************************************************************************/
#ifndef DISPLAY_H
#define DISPLAY_H

#include  "../main/main.h"
#include  "../timer/timer.h"          //    .

/*--------------------------------------------------------------------------------------------*/
/*     (PC).                                                        */
/*--------------------------------------------------------------------------------------------*/
#define   LCD_Control_Port_Off()    DDRC = PORTC = 0x00

/*--------------------------------------------------------------------------------------------*/
/*  A0  (/) (PC0).                                                       */
/*--------------------------------------------------------------------------------------------*/
// "1" -   , "0" -   .
#define   LCD_A0_PIN      C, 0

/*--------------------------------------------------------------------------------------------*/
/*  RW  (/) (PC1).                                                        */
/*--------------------------------------------------------------------------------------------*/
// "0" -  , "1" -  .
#define   LCD_RW_PIN      C, 1

/*--------------------------------------------------------------------------------------------*/
/*  E  () (PC2).                                                                 */
/*--------------------------------------------------------------------------------------------*/
//   - .
#define   LCD_E_PIN       C, 2

/*--------------------------------------------------------------------------------------------*/
/*  CS1  () (PC3).                                                             */
/*--------------------------------------------------------------------------------------------*/
//   - .
#define   LCD_CS1_PIN     C, 3

/*--------------------------------------------------------------------------------------------*/
/*  RES  () (PC5).                                                               */
/*--------------------------------------------------------------------------------------------*/
//   - .
#define   LCD_RES_PIN     C, 5

/*--------------------------------------------------------------------------------------------*/
/*    (    ) (PA).                            */
/*--------------------------------------------------------------------------------------------*/
#define   LCD_Data_Bus_to_Output()        DDRA = 0xff
#define   LCD_Data_Bus_To_Input()         DDRA = PORTA = 0x00
#define   LCD_Data_Bus_Off()              DDRA = PORTA = 0x00
#define   Put_Data_to_LCD_Data_Bus(data)  PORTA = (data)
#define   Get_Data_from_LCD_Data_Bus()    PINA

/*--------------------------------------------------------------------------------------------*/
/*     .                                                           */
/*--------------------------------------------------------------------------------------------*/
#define   MAX_WIDTH     128
#define   MAX_HEIGHT    64

/*--------------------------------------------------------------------------------------------*/
/*   .                                                           */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Display_Init(void);

/*--------------------------------------------------------------------------------------------*/
/*      .                                                     */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Display_Sleep()         { Write_Cmd_Display_OFF(); LCD_Control_Port_Off(); }

/*--------------------------------------------------------------------------------------------*/
/*      .                                                     */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Display_WakeUp()                  Display_Init()

/*--------------------------------------------------------------------------------------------*/
/*      .                                                 */
/* :     .                                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Cmd_to_LCD(IN BYTE cmd);

/*--------------------------------------------------------------------------------------------*/
/*       (Display_ON).                                 */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Display_ON()            Write_Cmd_to_LCD(0xaf)

/*--------------------------------------------------------------------------------------------*/
/*       (Display_OFF).                               */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Display_OFF()           Write_Cmd_to_LCD(0xae)

/*--------------------------------------------------------------------------------------------*/
/*           ,       */
/*     (Set_Start_Line).                                                   */
/* :     (0..63).                                                         */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Set_Start_Line(line)    Write_Cmd_to_LCD(0x40 + (line))

/*--------------------------------------------------------------------------------------------*/
/*         (Page_Addr_Set).              */
/* :     (0..7).                                                        */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Page_Addr_Set(page)     Write_Cmd_to_LCD(0xb0 + (page))

/*--------------------------------------------------------------------------------------------*/
/*         (Column_Addr_Set).             */
/* :     (0..127).                                                       */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Column_Addr_Set(col)      \
{                                             \
  Write_Cmd_to_LCD(0x10 + ((col) >> 4));      \
  Write_Cmd_to_LCD(0x00 + ((col) & 0x0f));    \
}

/*--------------------------------------------------------------------------------------------*/
/*         - (ADC_Normal).*/
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_ADC_Normal()            Write_Cmd_to_LCD(0xa0)

/*--------------------------------------------------------------------------------------------*/
/*         - (ADC_Reverse)*/
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_ADC_Reverse()           Write_Cmd_to_LCD(0xa1)

/*--------------------------------------------------------------------------------------------*/
/*          - "1"      */
/*       (Display_Normal).                              */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Display_Normal()        Write_Cmd_to_LCD(0xa6)

/*--------------------------------------------------------------------------------------------*/
/*          - "0"       */
/*       (Display_Reverse).                             */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Display_Reverse()       Write_Cmd_to_LCD(0xa7)

/*--------------------------------------------------------------------------------------------*/
/*         (All_Points_On).                */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_All_Points_On()         Write_Cmd_to_LCD(0xa4)

/*--------------------------------------------------------------------------------------------*/
/*         (All_Points_Off).              */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_All_Points_Off()        Write_Cmd_to_LCD(0xa5)

/*--------------------------------------------------------------------------------------------*/
/*           (LCD_Bias_Set). */
/* :      ("0" - 1/9, "1" - 1/7).                               */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_LCD_Bias_Set(n)         Write_Cmd_to_LCD(0xa2 + (n))

/*--------------------------------------------------------------------------------------------*/
/*       (LCD_Reset).                                     */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_LCD_Reset()             Write_Cmd_to_LCD(0xe2)

/*--------------------------------------------------------------------------------------------*/
/*           -       */
/* - (COM_Output_Scan_Direction_Normal).                                            */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Scan_Normal()           Write_Cmd_to_LCD(0xc0)

/*--------------------------------------------------------------------------------------------*/
/*           -        */
/* - (COM_Output_Scan_Direction_reverse).                                          */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Scan_Reverse()          Write_Cmd_to_LCD(0xc8)

/*--------------------------------------------------------------------------------------------*/
/*        (Power_Control_Set).            */
/* :        (booster),                 */
/*                   (voltage regulator),             */
/*                  (follower).                                */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Power_Control_Set(booster, regulator, follower)  \
  Write_Cmd_to_LCD(0x28 + ((booster) << 2) + ((regulator) << 1) + (follower))

/*--------------------------------------------------------------------------------------------*/
/*              */
/* V5 (V5_Voltage_Regulator_Internal_Resistor_Ratio).                                         */
/* :     ("0" - min  .. "7" - max ).                */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_V5_Res_Ratio(ratio)     Write_Cmd_to_LCD(0x20 + (ratio))

/*--------------------------------------------------------------------------------------------*/
/*          (Volume_Set).         */
/* :      ("1" - min .. "63" - max).                   */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Volume_Set(ratio)       \
{                                           \
  Write_Cmd_to_LCD(0x81);                   \
  Write_Cmd_to_LCD(0x00 + (ratio));         \
}

/*--------------------------------------------------------------------------------------------*/
/*               */
/*  booster (Booster_Ratio_Set).                                                        */
/* :     (0, 1, 3).                                           */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Booster_Ratio(ratio)    \
{                                           \
  Write_Cmd_to_LCD(0xf8);                   \
  Write_Cmd_to_LCD(0x00 + (ratio));         \
}

/*--------------------------------------------------------------------------------------------*/
/*      NOP (NOP).                                              */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_NOP()                   Write_Cmd_to_LCD(0xe3)

/*--------------------------------------------------------------------------------------------*/
/*   1     .                                          */
/* :     .                                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN BYTE data);

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                         */
/* :      ,   .                       */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN BYTE *pData, IN BYTE n);
void Write_Data_to_LCD(IN BYTE __flash *pData, IN BYTE n);

/*--------------------------------------------------------------------------------------------*/
/*   n      .                                    */
/* :    ,   .                                         */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN BYTE data, IN BYTE n);

/*--------------------------------------------------------------------------------------------*/
/*   .                                                                     */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void LCD_Clear(void);

/*--------------------------------------------------------------------------------------------*/
/*   1     .                                        */
/* :     .                                                          */
/* :   .                                                               */
/*--------------------------------------------------------------------------------------------*/
OUT BYTE Read_Data_from_LCD(void);

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                       */
/* :       ,   .                 */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Read_Data_from_LCD(OUT BYTE *pData, IN BYTE n);

/*--------------------------------------------------------------------------------------------*/
/*       .                                  */
/* :      (),                                  */
/*                 ,                                                  */
/*                    flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   DRAW_DIRECT         0
#define   DRAW_XOR            (1 << 0)
#define   DRAW_INVERTED       (1 << 1)
#define   DRAW_UNDERLINED     (1 << 2)
void Draw_Bitmap(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H, IN BYTE __flash *pData, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*     .                                       */
/* :         ,                 */
/*                   ,                                 */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Fill_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*      .                                            */
/* :      (),                                        */
/*                 .                                                  */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Put_Pixel(IN BYTE X, IN BYTE Y, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*      .                                      */
/* :        (),                               */
/*                 .                                                */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Line(IN BYTE X1, IN BYTE Y1, IN BYTE X2, IN BYTE Y2, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*   .                                                        */
/* :        ,                     */
/*                  .                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
inline void Draw_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H)
{
  Draw_Line(X, Y, X + W - 1, Y); Draw_Line(X, Y + H - 1, X + W - 1, Y + H - 1);
  Draw_Line(X, Y, X, Y + H - 1); Draw_Line(X + W - 1, Y, X + W - 1, Y + H - 1);
}

/*--------------------------------------------------------------------------------------------*/
/*     .                                        */
/* :         ,                     */
/*                   .                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Clr_Rect(x, y, w, h)  Fill_Rect((x), (y), (w), (h), DRAW_INVERTED)

/*--------------------------------------------------------------------------------------------*/
/*     .                                     */
/* :         ,                 */
/*                   .                                 */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Invert_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H);

/*--------------------------------------------------------------------------------------------*/
/*        .                      */
/* :    ,                                                                  */
/*                 (),                                      */
/*                   flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Draw_Char(ch, X, Y, font, flags)    \
  Draw_Bitmap((X), (Y), (font)->Width, (font)->Height, Get_Char_Data_Addr((ch), (font)), (flags))

/*--------------------------------------------------------------------------------------------*/
/*        .                          */
/* :    ,                                                                 */
/*                 (),                                      */
/*                   flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Text(IN char *str, IN BYTE X, IN BYTE Y, IN struct FONT __flash *font, IN BYTE flags = DRAW_DIRECT);
void Draw_Text(IN char __flash *str, IN BYTE X, IN BYTE Y, IN struct FONT __flash *font, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*      .                                           */
/* :     (   flash),                                               */
/*                   flash.                                     */
/* :     .                                                      */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Text_Width(str, font)       Str_Len((str)) * (font)->Width

/*--------------------------------------------------------------------------------------------*/
/*      .                                          */
/* :     (   flash),                                               */
/*                   flash,                                     */
/* :     .                                                     */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Text_Height(str, font)      (font)->Height

/*--------------------------------------------------------------------------------------------*/
/*      .                                         */
/* :        flash,                                     */
/* :    .                                                            */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Char_Width(font)            (font)->Width

/*--------------------------------------------------------------------------------------------*/
/*      .                                         */
/* :        flash,                                     */
/* :    .                                                            */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Char_Height(font)           (font)->Height

/*--------------------------------------------------------------------------------------------*/
/*          .    */
/* :    ,                                                                  */
/*                   flash,                                     */
/* :     .                                                */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Char_Data_Addr(ch, font)    \
  (font)->pData + (ch) * (font)->Width * ((((font)->Height % 8) != 0) ? (1 + (font)->Height / 8) : ((font)->Height / 8))

/*--------------------------------------------------------------------------------------------*/
/*   .                                                                 */
/*--------------------------------------------------------------------------------------------*/
struct  FONT
{
  BYTE  Height;         //    .
  BYTE  Width;          //    .
  BYTE  Count;          //      .
  BYTE __flash *pData;  //      .
};

/*--------------------------------------------------------------------------------------------*/
/*  6  8 .                                                                      */
/*--------------------------------------------------------------------------------------------*/
extern __flash struct FONT Font_6x8;
extern __flash BYTE Font_6x8_Data[];

/*--------------------------------------------------------------------------------------------*/
/*  7  10  .                                                              */
/*--------------------------------------------------------------------------------------------*/
extern __flash struct FONT Font_7x10_thin;
extern __flash BYTE Font_7x10_thin_Data[];

/*--------------------------------------------------------------------------------------------*/
/*  7  10  .                                                          */
/*--------------------------------------------------------------------------------------------*/
extern __flash struct FONT Font_7x10_bold;
extern __flash BYTE Font_7x10_bold_Data[];

/*--------------------------------------------------------------------------------------------*/
/*  12  16 .                                                                    */
/*--------------------------------------------------------------------------------------------*/
extern __flash struct FONT Font_12x16;
extern __flash BYTE Font_12x16_Data[];



/**********************************************************************************************/
#endif
